import { formatLines } from "../../utils";
import { dropConstraint } from "./dropConstraint";
import { parseConstraints } from "./shared";
function addConstraint(mOptions) {
  const _add = (tableName, constraintName, expression) => {
    const { constraints, comments } = typeof expression === "string" ? {
      constraints: [
        `${constraintName ? `CONSTRAINT ${mOptions.literal(constraintName)} ` : ""}${expression}`
      ],
      comments: []
    } : parseConstraints(
      tableName,
      expression,
      constraintName,
      mOptions.literal
    );
    const constraintStr = formatLines(constraints, "  ADD ");
    return [
      `ALTER TABLE ${mOptions.literal(tableName)}
${constraintStr};`,
      ...comments
    ].join("\n");
  };
  _add.reverse = (tableName, constraintName, options) => {
    if (constraintName === null) {
      throw new Error(
        "Impossible to automatically infer down migration for addConstraint without naming constraint"
      );
    }
    if (typeof options === "string") {
      throw new Error(
        "Impossible to automatically infer down migration for addConstraint with raw SQL expression"
      );
    }
    return dropConstraint(mOptions)(tableName, constraintName, options);
  };
  return _add;
}
export {
  addConstraint
};
